#include <cmath>

#include "waves.hh"

static const double PI256 = M_PI * 2 / TABSIZE;

static class Tabs
{
public:
	double s[TABSIZE];
	double c[TABSIZE];
	double t[TABSIZE];

	Tabs()
	{
		for(unsigned a=0; a<TABSIZE; ++a)
		{
			s[a] = std::sin(a * PI256);
			c[a] = std::cos(a * PI256);
			t[a] = std::tan(a * PI256);
		}
	}
} Tabs;

namespace mun {

inline double sin(double p)
{
	return Tabs.s[(static_cast<int> (p / PI256)) % TABSIZE];
}

inline double cos(double p)
{
	return Tabs.c[(static_cast<int> (p / PI256)) % TABSIZE];
}

inline double tan(double p)
{
	return Tabs.t[(static_cast<int> (p / PI256)) % TABSIZE];
}


static double wave0(double ang)
{
	// Normal sine
	
	return sin(ang);
}

static double wave1(double ang)
{
	// Negative part cropped
	
	register double tmp = sin(ang);
	return tmp > 0 ? tmp : 0;
}

static double wave2(double ang)
{
	// Rectified (negative made positive)
	
	register double tmp = sin(ang);
	return tmp >= 0 ? tmp : -tmp;
}

static double wave3(double ang)
{
	// Rectified (negative made positive)
	
	register double tmp = sin(ang);
	if(tmp <= 0.0)return 0.0;
	
	// Only the first quarter of each sine wave
	
	register double cosi = cos(ang);
	if(cosi <= 0.0)return 0.0;
	
	return tmp;
}

static double wave4(double ang)
{
	// Sine, even periods only
	
	register double tmp = sin(ang);
	if(tmp <= 0.0)return 0.0;
	return sin(ang*2);
}

static double wave5(double ang)
{
	// Abs-sine, even periods only
	
	register double tmp = sin(ang);
	if(tmp <= 0.0)return 0.0;

	tmp = sin(ang*2);
	return tmp >= 0 ? tmp : -tmp;
}

static double wave6(double ang)
{
	// Square
	
	register double tmp = sin(ang);
	
	return tmp > 0 ? 1 : -1;
}

static double wave7(double ang)
{
	// Derived square

	register double tmp = sin(ang*2);
	if(tmp <= 0.0)return 0.0;

	tmp = tan(ang);
	if(tmp > 1)return 1;
	if(tmp < -1)return -1;
	return tmp;
}

}

double (*waves[8])(double) =
{ mun::wave0, mun::wave1, mun::wave2, mun::wave3,
  mun::wave4, mun::wave5, mun::wave6, mun::wave7 };
